*! version 1.02 -- pbe -- 11/5/2010
program regfungible
version 10.1
  syntax [, theta(real .01) sets(integer 50) print PREfix(string) seed(string) ]
  /* adapted from the R functon fungible by Niels Waller, University of Minnesota */
  if "`e(cmd)'"~="regress" {
    display as err "regress command not found"
    exit
  }
  if e(df_m)<2 {
    display as err "regression model must have at least 2 predictors"
    display as err "3 or more predictors are much better"
    exit
  }
  if "`seed'"!="" {
    set seed `seed'
  }
  scalar thta=`theta'
  scalar sts=`sets'
  if `sets'>_N {
    set obs `sets'
  }
  if "`prefix'"=="" {
    local prefix "v_"
  }
  local cmd = e(cmdline)
  local cmd = subinstr("`cmd'","regress","corr",1)
  if "`print'"=="print" {
    display
    display as txt "Correlation matrix for raw data "
    `cmd'
  }
  else {
    quietly `cmd'
  }
 
  mata: fungible()
  
  display
  display as txt "Summary of fungible regresson weights"
  tabstat `prefix'*, stat(n mean p5 p25 p50 p75 p95)
 
end

version 10.1
mata:
void fungible()
{ 
  string scalar pre, strlist
  
  C = st_matrix("r(C)")
  RX  = C[2..rows(C),2..cols(C)]
  rxy = C[1, 2..cols(C)]
  RSQb  = st_numscalar("e(r2)")
  theta = st_numscalar("thta")
  sets = st_numscalar("sts")
  RSQa = RSQb - theta
  pre = st_local("prefix")
  print = st_local("print")
  r_yhata_yhatb = sqrt(1 - theta/RSQb)
  printf("{txt} \n\n OLS fungible regression weights analysis \n")
  printf("{txt} \n Original R2: RSQb = {res}%9.0g \n",RSQb )
  printf("{txt} Reduced R2:  RSQa = {res}%9.0g \n",RSQa )
  printf("{txt} theta = RSQb-RSQa = {res}%9.0g \n",theta )
  printf("{txt} r_yhata_yhatb     = {res}%9.0g \n",r_yhata_yhatb )
  
  NX = cols(RX)
  amatrix = J(1,NX,0)
  kmatrix = J(1,NX,0)
  V = .
  L = .
  
  /* OLS weights */
  b = invsym(RX)*rxy'
  r = r_yhata_yhatb
  
  eigensystem(RX, V, L)
  L = diag(L)
  Linv_sqrt = cholinv(sqrt(L)) 
  ustar   = V'*b
  ucircle = sqrt(L)*ustar
  u = ucircle/(sqrt(ucircle'*ucircle))
  r_y_yhatb = sqrt(b'*RX*b)
  mat = rnormal(NX,NX-1,0,1)
  
  /* generate U matrix via gram schmidt */
  p = cols(mat)
  n = rows(mat)
  oData = u

  for (i=1; i<=p; i++) {
     y = mat[.,i]
     x = oData[.,1..i]
     resid = y-x*(cholinv(x'*x)*(x'*y))
     oData = oData,resid 
  } 
  
  U = oData[.,2..(p+1)]
  d = diag(1:/sqrt(diag(U'*U)))
  U = U*d
  /* end gram schmidt */
  
  for (i=1; i<=sets; i++) {
    z = rnormal(1,NX-1,0,1)
    z = z/sqrt(z*z')
    
    k = r*u + U*z'*sqrt(1-r^2)    
    kstar = Linv_sqrt*k
    a = V*kstar
    
    /* scale a to minimize SSE_a */
    s = (rxy*a)/(a'*RX*a)
    a = s*a
    if (i==1) {
      printf("\n\nGenerating Alternate weights ...\n")
      r_yhata_yhatb = (a'*RX*b)/(sqrt(a'*RX*a)*sqrt(b'*RX*b))
    }    
    amatrix = amatrix \ a'
    kmatrix = kmatrix \ k'
  }
  amatrix = Re(amatrix[2..rows(amatrix),.])
  kmatrix = Re(kmatrix[2..rows(kmatrix),.])
    strlist = J(1,NX," ")
    for (i=1; i<=NX; i++) {
      strlist[1,i] = pre+strofreal(i)
    }
    newvars = st_addvar("float",strlist)
    st_store((1,rows(amatrix)),newvars,amatrix) 
    w=.; mi=.; ma=.;
  for (r=1; r<=NX; r++) {
    maxindex(amatrix[.,r],1,ma,w)
    minindex(amatrix[.,r],1,mi,w)
    if (r==1) {
      max = ma[1,1]
      min = mi[1,1]
    }
    if (r>1)  {
      max=max,ma[1,1]
      min=min,mi[1,1]
    }
  }
  printf("{txt} \n\n Standardized OLS regression weights \n")
  b'
  
  printf("{txt}\n Maximum fungible regression weights for each variable \n")
  amatrix[(max),.]

  printf("{txt}\n Minimum fungible regression weights for each variable \n")
  amatrix[(min),.]

  /* Compute Expected Moments */
  G = V*Linv_sqrt*U 
  esq = (1-r^2) 
  mn_a = r^2*b 
  Ezsq = 1/(NX-1)
  
  if (print=="print") {
    printf("{txt}\n Expected a \n") 
    mn_a
    cova = (r^2*r_y_yhatb^2)*Ezsq*esq*G*G'
    Dmat = diag(1:/sqrt(diag(cova)))
    cora = Dmat*cova*Dmat
    printf("\n Expected covariance matrix \n")
    cova
    printf("\n Expected correlation matrix \n")
    cora
  }
  
}
end
